@@ -1,4 +1,9 @@
 import combine_gtfs_feeds.cli.log_controller as log_controller
+try:
+    from .gtfs_schema import GTFS_Schema
+except:
+    from gtfs_schema import GTFS_Schema
+
 import pandas as pd
 import numpy as np
 import os as os
@@ -13,17 +18,60 @@ import zipfile
 class Combined_GTFS(object):
     file_list = ["agency", "trips", "stop_times", "stops", "routes", "shapes"]
 
-    def __init__(self, df_dict):
-        self.agency_df = df_dict["agency"]
-        self.routes_df = df_dict["routes"]
-        self.stops_df = df_dict["stops"]
-        self.stop_times_df = df_dict["stop_times"]
-        self.shapes_df = df_dict["shapes"]
-        self.trips_df = df_dict["trips"]
-        self.calendar_df = df_dict["calendar"]
+    def __init__(self, df_dict, output_dir):
+        # self.agency_df = df_dict["agency"]
+        self.output_dir = output_dir
+        self.agency_df = GTFS_Schema.Agency.validate(df_dict["agency"])
+        self.agency_df = self.agency_df[
+            [col for col in GTFS_Schema.agency_columns if col in self.agency_df.columns]
+        ]
 
-    def export_feed(self, dir):
-        dir = Path(dir)
+        # self.routes_df = df_dict["routes"]
+        self.routes_df = GTFS_Schema.Routes.validate(df_dict["routes"])
+        self.routes_df = self.routes_df[
+            [col for col in GTFS_Schema.routes_columns if col in self.routes_df.columns]
+        ]
+
+        # self.stops_df = df_dict["stops"]
+        self.stops_df = GTFS_Schema.Stops.validate(df_dict["stops"])
+        self.stops_df = self.stops_df[
+            [col for col in GTFS_Schema.stops_columns if col in self.stops_df.columns]
+        ]
+
+        # self.stop_times_df = df_dict["stop_times"]
+        self.stop_times_df = GTFS_Schema.Stop_Times.validate(df_dict["stop_times"])
+        self.stop_times_df = self.stop_times_df[
+            [
+                col
+                for col in GTFS_Schema.stop_times_columns
+                if col in self.stop_times_df.columns
+            ]
+        ]
+
+        # self.shapes_df = df_dict["shapes"]
+        self.shapes_df = GTFS_Schema.Shapes.validate(df_dict["shapes"])
+        self.shapes_df = self.shapes_df[
+            [col for col in GTFS_Schema.shapes_columns if col in self.shapes_df.columns]
+        ]
+
+        # self.trips_df = df_dict["trips"]
+        self.trips_df = GTFS_Schema.Trips.validate(df_dict["trips"])
+        self.trips_df = self.trips_df[
+            [col for col in GTFS_Schema.trips_columns if col in self.trips_df.columns]
+        ]
+
+        # self.calendar_df = df_dict["calendar"]
+        self.calendar_df = GTFS_Schema.Calendar.validate(df_dict["calendar"])
+        self.calendar_df = self.calendar_df[
+            [
+                col
+                for col in GTFS_Schema.calendar_columns
+                if col in self.calendar_df.columns
+            ]
+        ]
+
+    def export_feed(self):
+        dir = Path(self.output_dir)
         self.agency_df.to_csv(dir / "agency.txt", index=None)
         self.routes_df.to_csv(dir / "routes.txt", index=None)
         self.stops_df.to_csv(dir / "stops.txt", index=None)
@@ -94,7 +142,10 @@ def create_id(df, feed, id_column):
     Changes id_column by prepending each value with
     the feed parameter.
     """
-    df[id_column] = feed + "_" + df[id_column].astype(str)
+    # df[id_column] = feed + "_" + df[id_column].astype(str)
+    df[id_column] = np.where(
+        ~df[id_column].isnull(), feed + "_" + df[id_column].astype(str), ""
+    )
     return df
 
 
@@ -316,18 +367,21 @@ def run(args):
     a single feed.
     """
 
-    logger = log_controller.setup_custom_logger("main_logger", args.output_dir)
-    logger.info("------------------combine_gtfs_feeds Started----------------")
+    #logger = log_controller.setup_custom_logger("main_logger", args.output_dir)
+    #logger.info("------------------combine_gtfs_feeds Started----------------")
 
-    feeds = combine(args.gtfs_dir, args.output_dir, args.service_date, logger)
+    feeds = combine(args.gtfs_dir, args.service_date, args.output_dir)
 
-    feeds.export_feed(args.output_dir)
+    feeds.export_feed()
 
-    logger.info("Finished running combine_gtfs_feeds")
+    #logger.info("Finished running combine_gtfs_feeds")
     # sys.exit()
+    print ("Finished running combine_gtfs_feeds")
 
 
-def combine(gtfs_dir, output_dir, service_date, logger):
+def combine(gtfs_dir, service_date, output_dir):
+    logger = log_controller.setup_custom_logger("main_logger", output_dir)
+    logger.info("------------------combine_gtfs_feeds Started----------------")
     output_loc = output_dir
 
     if not os.path.isdir(output_loc):
@@ -416,6 +470,8 @@ def combine(gtfs_dir, output_dir, service_date, logger):
         routes = read_gtfs(full_path, "routes.txt", zipped)
         shapes = read_gtfs(full_path, "shapes.txt", zipped)
         agency = read_gtfs(full_path, "agency.txt", zipped)
+        if "agency_id" not in routes.columns:
+            routes["agency_id"] = agency["agency_id"][0]
 
         # create new IDs
         trips = create_id(trips, feed, "trip_id")
@@ -492,7 +548,9 @@ def combine(gtfs_dir, output_dir, service_date, logger):
             df = pd.concat([df, feed_dict[feed][file_name]])
         combined_feed_dict[file_name] = df
 
-    return Combined_GTFS(combined_feed_dict)
+    #logger.info("Finished running combine_gtfs_feeds")
+
+    return Combined_GTFS(combined_feed_dict, output_dir)
 
 
 if __name__ == "__main__":
